(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('react')) :
  typeof define === 'function' && define.amd ? define(['react'], factory) :
  (global.ReactIntersectionObserver = factory(global.React));
}(this, (function (React) { 'use strict';

  var INSTANCE_MAP = new Map();
  var OBSERVER_MAP = new Map();

  /**
   * Monitor element, and trigger callback when element becomes visible
   * @param element {HTMLElement}
   * @param callback {Function} Called with inView
   * @param options {Object} InterSection observer options
   * @param options.threshold {Number} Number between 0 and 1, indicating how much of the element should be visible before triggering
   * @param options.root {HTMLElement} It should have a unique id or data-intersection-id in order for the Observer to reused.
   * @param options.rootMargin {String} The CSS margin to apply to the root element.
   * @param rootId {String} Unique identifier for the root element, to enable reusing the IntersectionObserver
   */
  function observe(element, callback) {
    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {
      threshold: 0
    };
    var rootId = arguments[3];
    var root = options.root,
        rootMargin = options.rootMargin;

    var threshold = options.threshold || 0;
    if (!element || !callback) return;
    var observerId = rootMargin ? threshold.toString() + "_" + rootMargin : "" + threshold.toString();

    if (root) {
      observerId = rootId ? rootId + "_" + observerId : null;
    }

    var observerInstance = observerId ? OBSERVER_MAP.get(observerId) : null;
    if (!observerInstance) {
      observerInstance = new IntersectionObserver(onChange, options);
      if (observerId) OBSERVER_MAP.set(observerId, observerInstance);
    }

    var instance = {
      callback: callback,
      visible: false,
      options: options,
      observerId: observerId,
      observer: !observerId ? observerInstance : undefined
    };

    INSTANCE_MAP.set(element, instance);

    observerInstance.observe(element);

    return instance;
  }

  /**
   * Stop observing an element. If an element is removed from the DOM or otherwise destroyed,
   * make sure to call this method.
   * @param element {HTMLElement}
   */
  function unobserve(element) {
    if (!element) return;
    var instance = INSTANCE_MAP.get(element);

    if (instance) {
      var _observerId = instance.observerId,
          _observer = instance.observer;

      var observerInstance = _observerId ? OBSERVER_MAP.get(_observerId) : _observer;

      if (observerInstance) {
        // $FlowFixMe - the interface in bom.js is wrong. Spec should accept the element.
        observerInstance.unobserve(element);
      }

      // Check if we are still observing any elements with the same threshold.
      var itemsLeft = false;
      if (_observerId) {
        INSTANCE_MAP.forEach(function (item, key) {
          if (item && item.observerId === _observerId && key !== element) {
            itemsLeft = true;
          }
        });
      }

      if (observerInstance && !itemsLeft) {
        // No more elements to observe for threshold, disconnect observer
        observerInstance.disconnect();
        OBSERVER_MAP.delete(_observerId);
      }

      // Remove reference to element
      INSTANCE_MAP.delete(element);
    }
  }

  function onChange(changes) {
    changes.forEach(function (intersection) {
      var isIntersecting = intersection.isIntersecting,
          intersectionRatio = intersection.intersectionRatio,
          target = intersection.target;

      var instance = INSTANCE_MAP.get(target);

      // Firefox can report a negative intersectionRatio when scrolling.
      if (instance && intersectionRatio >= 0) {
        var _options = instance.options;

        var _inView = false;

        if (Array.isArray(_options.threshold)) {
          // If threshold is an array, check if any of them intersects. This just triggers the onChange event multiple times.
          _inView = _options.threshold.some(function (threshold) {
            return instance.visible ? intersectionRatio > threshold : intersectionRatio >= threshold;
          });
        } else if (_options.threshold !== undefined) {
          // Trigger on 0 ratio only when not visible. This is fallback for browsers without isIntersecting support
          _inView = instance.visible ? intersectionRatio > _options.threshold : intersectionRatio >= _options.threshold;
        }

        if (isIntersecting !== undefined) {
          // If isIntersecting is defined, ensure that the element is actually intersecting.
          // Otherwise it reports a threshold of 0
          _inView = _inView && isIntersecting;
        }

        instance.visible = _inView;
        if (instance.callback) {
          instance.callback(_inView);
        }
      }
    });
  }

  /**
   * Copyright (c) 2013-present, Facebook, Inc.
   *
   * This source code is licensed under the MIT license found in the
   * LICENSE file in the root directory of this source tree.
   */

  var invariant = function(condition, format, a, b, c, d, e, f) {
    {
      if (format === undefined) {
        throw new Error('invariant requires an error message argument');
      }
    }

    if (!condition) {
      var error;
      if (format === undefined) {
        error = new Error(
          'Minified exception occurred; use the non-minified dev environment ' +
          'for the full error message and additional helpful warnings.'
        );
      } else {
        var args = [a, b, c, d, e, f];
        var argIndex = 0;
        error = new Error(
          format.replace(/%s/g, function() { return args[argIndex++]; })
        );
        error.name = 'Invariant Violation';
      }

      error.framesToPop = 1; // we don't care about invariant's own frame
      throw error;
    }
  };

  var invariant_1 = invariant;

  var classCallCheck = function (instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  };

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  var inherits = function (subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  };

  var objectWithoutProperties = function (obj, keys) {
    var target = {};

    for (var i in obj) {
      if (keys.indexOf(i) >= 0) continue;
      if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
      target[i] = obj[i];
    }

    return target;
  };

  var possibleConstructorReturn = function (self, call) {
    if (!self) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return call && (typeof call === "object" || typeof call === "function") ? call : self;
  };

  /**
   * Monitors scroll, and triggers the children function with updated props
   *
   <Observer>
   {inView => (
     <h1>{`${inView}`}</h1>
   )}
   </Observer>
   */
  var Observer = function (_React$Component) {
    inherits(Observer, _React$Component);

    function Observer() {
      var _temp, _this, _ret;

      classCallCheck(this, Observer);

      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return _ret = (_temp = (_this = possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args))), _this), _this.state = {
        inView: false
      }, _this.node = null, _this.handleNode = function (node) {
        if (_this.node) unobserve(_this.node);
        _this.node = node;
        _this.observeNode();
      }, _this.handleChange = function (inView) {
        _this.setState({ inView: inView });
        if (_this.props.onChange) {
          _this.props.onChange(inView);
        }
      }, _temp), possibleConstructorReturn(_this, _ret);
    }

    Observer.prototype.componentDidMount = function componentDidMount() {
      if (typeof this.props.render === 'function') {
        invariant_1(this.node, 'react-intersection-observer: No DOM node found. Make sure you forward "ref" to the root DOM element you want to observe, when using render prop.');
      }
    };

    Observer.prototype.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
      // If a IntersectionObserver option changed, reinit the observer
      if (prevProps.rootMargin !== this.props.rootMargin || prevProps.root !== this.props.root || prevProps.threshold !== this.props.threshold) {
        unobserve(this.node);
        this.observeNode();
      }

      if (prevState.inView !== this.state.inView) {
        if (this.state.inView && this.props.triggerOnce) {
          unobserve(this.node);
          this.node = null;
        }
      }
    };

    Observer.prototype.componentWillUnmount = function componentWillUnmount() {
      if (this.node) {
        unobserve(this.node);
        this.node = null;
      }
    };

    Observer.prototype.observeNode = function observeNode() {
      if (!this.node) return;
      var _props = this.props,
          threshold = _props.threshold,
          root = _props.root,
          rootMargin = _props.rootMargin,
          rootId = _props.rootId;

      observe(this.node, this.handleChange, {
        threshold: threshold,
        root: root,
        rootMargin: rootMargin
      }, rootId);
    };

    Observer.prototype.render = function render() {
      var _props2 = this.props,
          children = _props2.children,
          render = _props2.render,
          tag = _props2.tag,
          triggerOnce = _props2.triggerOnce,
          threshold = _props2.threshold,
          root = _props2.root,
          rootId = _props2.rootId,
          rootMargin = _props2.rootMargin,
          props = objectWithoutProperties(_props2, ['children', 'render', 'tag', 'triggerOnce', 'threshold', 'root', 'rootId', 'rootMargin']);
      var inView = this.state.inView;


      if (typeof render === 'function') {
        return render({ inView: inView, ref: this.handleNode });
      }

      return React.createElement(tag, _extends({}, props, {
        ref: this.handleNode
      }),
      // If children is a function, render it with the current inView status.
      // Otherwise always render children. Assume onChange is being used outside, to control the the state of children.
      typeof children === 'function' ? children(inView) : children);
    };

    return Observer;
  }(React.Component);

  Observer.defaultProps = {
    tag: 'div',
    threshold: 0,
    triggerOnce: false
  };

  return Observer;

})));
